"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RJSVM_Builder = void 0;
const types_1 = require("./types");
const xrpl_1 = require("xrpl");
const xrpio_1 = require("xrpio");
const dataparser_1 = require("../../util/dataparser");
const protocol_constants_1 = require("../../util/protocol.constants");
class RJSVM_Builder {
    static from(Base, defs) {
        return class RJSVM_Runnable extends Base {
            config;
            sync_block_height = -1;
            rippleApi;
            xrpIO;
            definitions;
            constructor(config) {
                super();
                this.config = config;
                if (!protocol_constants_1.XRP_ADDRESS.test(this.owner)) {
                    throw new Error(`Inavlid owner address ${this.owner}`);
                }
                this.definitions = Object.freeze(defs);
                Object.entries(this.definitions).forEach(([k, v]) => {
                    this[k] = (env, ...args) => v.implementation.apply(this, [env, ...args]);
                });
            }
            connect = async () => {
                this.rippleApi = new xrpl_1.Client(this.config.rippleNode);
                await this.rippleApi.connect();
                this.xrpIO = new xrpio_1.xrpIO(this.config.rippleNode);
                await this.xrpIO.connect();
                await this.sync();
                this.rippleApi.on('ledgerClosed', (ledger) => {
                    /*
                    console.log("---- Ledger ----")
                    console.log("index",ledger.ledger_index)
                    console.log("hash", ledger.ledger_hash)
                    console.log("---- /Ledger ----")
                    */
                });
                await this.rippleApi.request({
                    command: 'subscribe',
                    streams: ['ledger']
                });
            };
            handlePayload = async (tx, payload) => {
                if (!(payload.endpoint in this.definitions)) {
                    return;
                }
                const endpointDef = this.definitions[payload.endpoint];
                if (endpointDef.visibility === 'owner' && tx.Account !== this.owner) {
                    console.log(`owner restricted endpoint "${payload.endpoint}" called from ${tx.hash}. But ${tx.Account} != ${this.owner}`);
                    return;
                }
                if (endpointDef.fee && Number(tx.Amount) < endpointDef.fee) {
                    console.log(`Insufficient fee ${tx.hash}. Required ${endpointDef.fee}, was ${tx.Amount}`);
                    return;
                }
                try {
                    const data = await this.xrpIO.treeRead([payload.data], tx.Account);
                    const jsonData = JSON.parse(data);
                    endpointDef.parameterSchema.parse(jsonData);
                    this[payload.endpoint].apply(this, [tx, jsonData]);
                }
                catch (err) {
                    console.log(err);
                    return;
                }
            };
            parseMemos = async (tx, memos) => {
                memos
                    .map((memo) => {
                    if (!memo.Memo || !memo.Memo.MemoData) {
                        return;
                    }
                    try {
                        const data = dataparser_1.DataParser.hex_to_ascii(memo.Memo.MemoData);
                        return JSON.parse(data);
                    }
                    catch (e) {
                        return;
                    }
                })
                    .filter((data) => data != undefined)
                    .forEach(payload => {
                    try {
                        const parsedPayload = types_1.payloadSchema.parse(payload);
                        this.handlePayload(tx, parsedPayload);
                    }
                    catch (e) {
                    }
                });
            };
            sync = async (marker) => {
                //If `forward` is set to true, returns values indexed with the oldest ledger first. 
                //Otherwise, the results are indexed with the newest ledger first. 
                //(Each page of results may not be internally ordered, but the pages are overall ordered.)
                const resp = await this.rippleApi.request({
                    command: "account_tx",
                    account: this.config.listeningAddress,
                    forward: true,
                    ledger_index_min: this.sync_block_height,
                    marker: marker,
                });
                //results may not be ordered, so sort them
                const raw_txs = resp.result.transactions
                    .sort((a, b) => a.tx.ledger_index - b.tx.ledger_index)
                    .map((entry) => entry.tx);
                //apply any state changes
                await Promise.all(raw_txs.map((tx) => this.parseMemos(tx, tx.Memos)));
                //if marker is present the result is paginated.
                //re-run the same request with the marker included
                if (resp.result.marker) {
                    await this.sync(resp.result.marker);
                }
                else {
                    //presence of no marker means we caught up to current block height
                    this.sync_block_height = resp.result.ledger_index_max + 1;
                    //schedule the next sync
                    setTimeout(this.sync, 10000);
                }
            };
        };
    }
}
exports.RJSVM_Builder = RJSVM_Builder;
