"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.xrpIO = void 0;
const types_1 = require("../util/types");
const xrpl_1 = require("xrpl");
const zlib = __importStar(require("zlib"));
const util = __importStar(require("util"));
const protocol_constants_1 = require("../util/protocol.constants");
const errors_1 = require("../util/errors");
const compressB64 = async (data) => (await util.promisify(zlib.deflate)(Buffer.from(data, 'utf-8'))).toString('base64');
const decompressB64 = async (data) => (await util.promisify(zlib.inflate)(Buffer.from(data, 'base64'))).toString('utf-8');
const hexDecode = (str) => Buffer.from(str, 'hex').toString('utf8');
const hexEncode = (str) => Buffer.from(str, 'utf8').toString('hex').toUpperCase();
const chunkString = (str, length) => str.match(new RegExp('.{1,' + length + '}', 'gs'));
const genRandHex = size => [...Array(size)].map(() => Math.floor(Math.random() * 16).toString(16)).join('');
const PAYLOAD_SIZE = 925;
const XRP_PER_DROP = 0.000001;
const DROP_PER_XRP = 1000000;
class xrpIO {
    server;
    options;
    api;
    constructor(server, options = types_1.defaultOptions) {
        this.server = server;
        this.options = options;
        this.options.debug = options.debug ? Boolean(options.debug) : types_1.defaultOptions.debug;
        this.options.connectionTimeout = options.connectionTimeout ? Number(options.connectionTimeout) : types_1.defaultOptions.connectionTimeout;
        this.options.readMaxRetry = options.readMaxRetry ? Number(options.readMaxRetry) : types_1.defaultOptions.readMaxRetry;
        this.options.readRetryTimeout = options.readRetryTimeout ? Number(options.readRetryTimeout) : types_1.defaultOptions.readRetryTimeout;
        this.api = new xrpl_1.Client(server, {
            connectionTimeout: this.options.connectionTimeout
        });
    }
    async connect() {
        if (!this.api.isConnected())
            await this.api.connect();
    }
    async disconnect() {
        try {
            await this.api.disconnect();
        }
        catch (e) {
            console.log("DISCONNECT ERROR", e);
        }
    }
    async cloneApi() {
        let _api = new xrpl_1.Client(this.server, {
            connectionTimeout: this.options.connectionTimeout
        });
        while (!_api.isConnected()) {
            try {
                await _api.connect();
                return _api;
            }
            catch (e) {
                this.dbg('CLONEAPI ERR', 'Connection failed', String(e['message']));
                await _api.disconnect();
                _api = new xrpl_1.Client(this.server, {
                    connectionTimeout: this.options.connectionTimeout
                });
            }
        }
    }
    async sendPayment(data, to, secret, sequence, amount = "1") {
        const wallet = xrpl_1.Wallet.fromSecret(secret);
        this.dbg("Sending payment", wallet.address, '->', to);
        const _api = await this.cloneApi();
        try {
            const payment = await _api.autofill({
                TransactionType: 'Payment',
                Account: wallet.address,
                Destination: to,
                Sequence: sequence,
                Amount: amount,
                Memos: [{
                        Memo: {
                            MemoData: hexEncode(data.data || ""),
                            MemoFormat: hexEncode(data.format || ""),
                            MemoType: hexEncode(data.type || "")
                        }
                    }]
            });
            const response = await _api.submitAndWait(payment, { wallet });
            await _api.disconnect();
            this.dbg("Tx finalized", response.result.hash, response.result.Sequence);
            return response;
        }
        catch (error) {
            this.dbg("SENDPAYMENT ERROR", error);
            await _api.disconnect();
            throw error;
        }
    }
    async writeRaw(data, to, secret, sequence, amount = "1") {
        this.dbg("Writing data", data);
        const tx = await this.sendPayment(data, to, secret, sequence, amount);
        return tx.result.hash;
    }
    async getTransaction(hash, retry = 0) {
        this.dbg("Getting Tx", hash);
        try {
            return await this.api.request({
                command: 'tx',
                transaction: hash,
            });
        }
        catch (e) {
            this.dbg(e);
            if (this.options.readMaxRetry != -1) {
                if (retry >= this.options.readMaxRetry)
                    console.error("Retry limit exceeded for", hash, ". This is an irrecoverable error");
                throw e;
            }
            await new Promise(res => setTimeout(res, this.options.readRetryTimeout));
            return await this.getTransaction(hash, retry + 1);
        }
    }
    async readRaw(hash, verifyOwner) {
        if (!protocol_constants_1.NON_ZERO_TX_HASH.test(hash)) {
            throw (0, errors_1.ERR_BAD_TX_HASH)(hash);
        }
        const tx = await this.getTransaction(hash);
        if (verifyOwner && tx.result.Account != verifyOwner) {
            throw (0, errors_1.ERR_NO_VERIFY_OWNER)(hash, tx.result.Account, verifyOwner);
        }
        const memo = tx.result.Memos[0].Memo;
        const memoParsed = {
            data: hexDecode(memo.MemoData),
            format: hexDecode(memo.MemoFormat),
            type: hexDecode(memo.MemoType)
        };
        this.dbg(hash, "data", memoParsed);
        return memoParsed;
    }
    async treeWrite(data, to, secret, format = 'L') {
        const wallet = xrpl_1.Wallet.fromSecret(secret);
        data = await compressB64(data);
        const chunks = chunkString(data, PAYLOAD_SIZE);
        const latestSequence = await this.getAccountSequence(wallet.address);
        const hashes = await Promise.all(Object.entries(chunks).map(([i, chunk]) => this.writeRaw({ data: chunk, format: format }, to, secret, latestSequence + Number(i))));
        if (hashes.length === 1) {
            return hashes[0];
        }
        return await this.treeWrite(JSON.stringify(hashes), to, secret, 'N');
    }
    async treeRead(hashes, verifyOwner) {
        const bad_hash = hashes.find(hash => !protocol_constants_1.NON_ZERO_TX_HASH.test(hash));
        if (bad_hash)
            throw (0, errors_1.ERR_BAD_TX_HASH)(bad_hash);
        const memos = await Promise.all(hashes.map(hash => this.readRaw(hash, verifyOwner)));
        const payload = await decompressB64(memos.map(memo => memo.data).join(''));
        if (memos.some(memo => memo.format === 'N')) {
            return await this.treeRead(JSON.parse(payload), verifyOwner);
        }
        return payload;
    }
    async getAccountSequence(address) {
        this.dbg("Getting acc info for", address);
        const accountInfo = await this.api.request({
            command: 'account_info',
            account: address,
            strict: true,
        });
        this.dbg("Got account_info", accountInfo);
        return Number(accountInfo.result.account_data.Sequence);
    }
    async estimateFee(data, denomination = 'DROPS', cost = 0) {
        data = await compressB64(data);
        const chunks = chunkString(data, PAYLOAD_SIZE);
        if (chunks.length === 1) {
            return (denomination === "DROPS" ? (cost + 1) : this.dropsToXrp(cost + 1));
        }
        return this.estimateFee(JSON.stringify(chunks.map(_ => genRandHex(64))), denomination, cost + chunks.length);
    }
    xrpToDrops(xrp) {
        return xrp * DROP_PER_XRP;
    }
    dropsToXrp(drops) {
        return drops * XRP_PER_DROP;
    }
    dbg(...args) {
        if (this.options.debug) {
            console.log.apply(console, args);
        }
    }
}
exports.xrpIO = xrpIO;
