import { PaymentTx_T, ParameterizedFunction, Payload, RJSVM, RJSVM_Config, RJSVM_Endpoint, RJSVM_Implementations, payloadSchema, Function_Map, Endpoints_Of, State_Of, Generic_Ctor_ReturnType } from "./types"
import { Client as Xrpl } from 'xrpl';
import { xrpIO } from 'xrpio';
import { DataParser } from "../../util/dataparser";
import { XRP_ADDRESS } from "../../util/protocol.constants";


export abstract class RJSVM_Builder {
    static from<
        Base_Ctor extends (new (...args: any) => RJSVM<State_T, Endpoints_T>) | (abstract new (...args: any) => RJSVM<State_T, Endpoints_T>),
        Impl extends RJSVM_Implementations<any, Endpoints_T>,
        Endpoints_T extends Function_Map = Endpoints_Of<Generic_Ctor_ReturnType<Base_Ctor>>,
        State_T = State_Of<Generic_Ctor_ReturnType<Base_Ctor>>
    >(Base: Base_Ctor, defs: Impl): (new (config: RJSVM_Config) => RJSVM<State_T, Endpoints_T>){
        return <any> class RJSVM_Runnable extends (Base as any){

            sync_block_height = -1

            private rippleApi: Xrpl
            private xrpIO: xrpIO
            public readonly definitions: Impl

            constructor(
                public readonly config: RJSVM_Config
            ) {
                super()

                
                if(!XRP_ADDRESS.test(this.owner)){
                    throw new Error(`Inavlid owner address ${this.owner}`)
                }
                

                this.definitions = Object.freeze(defs)
                Object.entries(this.definitions).forEach(([k, v]) => {
                    this[k] = (env: PaymentTx_T, ...args: any) => (v.implementation as ParameterizedFunction).apply(this as any, [env, ...args])
                })
            }

            public connect = async () => {
                this.rippleApi = new Xrpl(this.config.rippleNode)
                await this.rippleApi.connect()

                this.xrpIO = new xrpIO(this.config.rippleNode);
                await this.xrpIO.connect()
                await this.sync()

                this.rippleApi.on('ledgerClosed', (ledger: any) => {
                    /*
                    console.log("---- Ledger ----")
                    console.log("index",ledger.ledger_index)
                    console.log("hash", ledger.ledger_hash)
                    console.log("---- /Ledger ----")
                    */
                })
                await this.rippleApi.request({
                    command: 'subscribe',
                    streams: ['ledger']
                })
            }

            private handlePayload = async (tx: PaymentTx_T, payload: Payload) => {
                if (!(payload.endpoint in this.definitions)) {
                    return
                }

                const endpointDef: RJSVM_Endpoint<RJSVM, any> = this.definitions[payload.endpoint]
 

                if(endpointDef.visibility === 'owner' && tx.Account !== this.owner){
                    console.log(`owner restricted endpoint "${payload.endpoint}" called from ${tx.hash}. But ${tx.Account} != ${this.owner}`)
                    return
                }

                if(endpointDef.fee && Number(tx.Amount) < endpointDef.fee){
                    console.log(`Insufficient fee ${tx.hash}. Required ${endpointDef.fee}, was ${tx.Amount}`)
                    return
                }
                
                try {
                    const data = await this.xrpIO.treeRead([payload.data], tx.Account)
                    const jsonData = JSON.parse(data)
                    endpointDef.parameterSchema.parse(jsonData)
                    this[payload.endpoint].apply(this, [tx, jsonData])
                } catch (err) {
                    console.log(err)
                    return
                }
            }

            private parseMemos = async (tx:PaymentTx_T, memos: any[]) => {
                memos
                    .map((memo: any) => {
                        if (!memo.Memo || !memo.Memo.MemoData) {
                            return
                        }
                        try {
                            const data = DataParser.hex_to_ascii(memo.Memo.MemoData)
                            return JSON.parse(data)
                        } catch (e) {
                            return
                        }
                    })
                    .filter((data: string) => data != undefined)
                    .forEach(payload => {
                        try {
                            const parsedPayload = payloadSchema.parse(payload)
                            this.handlePayload(tx, parsedPayload)
                        } catch (e) {
                        }
                    })
            }

            private sync = async (marker?: any): Promise<void> => {
                //If `forward` is set to true, returns values indexed with the oldest ledger first. 
                //Otherwise, the results are indexed with the newest ledger first. 
                //(Each page of results may not be internally ordered, but the pages are overall ordered.)

                const resp = await this.rippleApi.request({
                    command: "account_tx",
                    account: this.config.listeningAddress,
                    forward: true, 
                    ledger_index_min: this.sync_block_height,
                    marker: marker,
                })

                //results may not be ordered, so sort them
                const raw_txs = resp.result.transactions
                    .sort((a,b) => a.tx.ledger_index - b.tx.ledger_index)
                    .map((entry: any) => entry.tx)
                
                //apply any state changes
                await Promise.all(raw_txs.map((tx: PaymentTx_T) => this.parseMemos(tx, tx.Memos)))
                
                //if marker is present the result is paginated.
                //re-run the same request with the marker included
                if(resp.result.marker){
                    await this.sync(resp.result.marker)
                }else{
                    //presence of no marker means we caught up to current block height
                    this.sync_block_height = resp.result.ledger_index_max + 1
                    //schedule the next sync
                    setTimeout(this.sync, 10000)
                }
            }
        }
    }
}